-- You may use and modify this file free
-- without deleting this comment group
-- Copyright (c) 1998 Dimitriy Anisimkov
-- e-mail anisimkov@yahoo.com
-- phone +7 (381)-2-217774
with Ada.Unchecked_Deallocation;
package body Dynamic_Controlled is

   function Create(Acc : in Static) return Dynamic is
   begin
     return (Auto with refer=> New Static'(Acc));
   end;
   
   NullVal : aliased Static := NullValue;
   procedure Initialize(Object : in out Dynamic) is 
   begin
     Object.Refer := NullVal'Access;
   end;
   
   procedure Adjust(Object : in out Dynamic) is
   begin
      if Object.Refer /= NullVal'Access then
        Object.Refer := new Static'(Object.Refer.all);
      end if;
   end Adjust;
      
   procedure Finalize(Object : in out Dynamic) is
   procedure Free is new Ada.Unchecked_Deallocation(Static, Static_Access);
   begin
      if Object.Refer /= NullVal'Access then
        Free(Object.Refer);
      end if;
   end Finalize;

   function Binary_Operator(D1,D2 : in Dynamic) return Dynamic is
   begin
     return Create(Operator(D1.Refer.All,D2.Refer.All));
   end;
   
   function Compare(D1,D2 : in Dynamic) return Boolean is
   begin
     return Comp(D1.Refer.All,D2.Refer.All);
   end;

   function AlienSecond(A : Dynamic;S : Alien) return Dynamic is
   begin
     return Create(AlienSecondParam(A.Refer.All,S));
   end;

   function Unary(A : Dynamic) return Dynamic is
   begin
     return Create(Unary_Stat(A.Refer.All));
   end;
   
   function GetAccess(from : Dynamic) return Static_Access is
   begin
     return From.Refer;
   end;

end;