-- Operations with fractional numbers
-- Copyright (c) 1998 Dmitriy Anisimkov
-- You may free use and modify this file 
-- without deleting this comment group
-- e-mail anisimkov@yahoo.com
-- phone +7 (381)-2217774
generic
   type Comp is private;
   NilComp : Comp;
   OneComp : Comp;
   with function "+"(E1,E2 : Comp) return Comp;
   with function "-"(E1,E2 : Comp) return Comp;
   with function "*"(E1,E2 : Comp) return Comp;
   with function "/"(E1,E2 : Comp) return Comp;
   with function "rem"(E1,E2 : Comp) return Comp;
   with function "abs"(E : Comp) return Comp;
   with function "-"(E : Comp) return Comp;
   with function ">"(E1,E2 : Comp) return Boolean;
   with function "<"(E1,E2 : Comp) return Boolean;
   with function ">="(E1,E2 : Comp) return Boolean;
   with function "<="(E1,E2 : Comp) return Boolean;
   with function "="(E1,E2 : Comp) return Boolean;
package Rational is

   type TRational is record
      Numer : Comp;
      Denom : Comp;
   end record;

function ">"(r1,r2 : TRational) return Boolean;
function "<"(r1,r2 : TRational) return Boolean;
function ">="(r1,r2 : TRational) return Boolean;
function "<="(r1,r2 : TRational) return Boolean;
-- this renaming only to avoid bug founded in AONIX build 7.1.424
function Equal4Aonix(R1,R2 : TRational) return Boolean;
function "="(r1,r2 : TRational) return Boolean renames Equal4Aonix;

  function "/"(Num,Denom : Comp) return TRational;
   function Inverse(R : TRational) return TRational;
   function MulRC(R : TRational;C : Comp) return TRational;
   function "*"(R1,R2 : TRational) return TRational;
   function "/"(R1,R2 : TRational) return TRational;
   function "-"(R : TRational) return TRational;
   function Fract(R : TRational) return TRational;
   function "+"(R1,R2 : TRational) return TRational;
   function "-"(R1,R2 : TRational) return TRational;
   function SumRC(R : TRational;C : Comp) return TRational;
   function Trunc(R : TRational) return TRational;
   function Euclide(A,B : Comp;Bl : Boolean) return TRational;
   function GreatestDivisor(C1,C2 : Comp) return Comp;
   function LowestFactor(C1,C2 : Comp) return Comp;
   function MinMembBetw(R1,R2 : TRational) return TRational;

   NegativeOne : constant TRational := (Numer => -OneComp,Denom => OneComp);
   One : constant TRational := (Numer => OneComp,Denom => OneComp);
   Nil : constant TRational := (Numer => NilComp,Denom => OneComp);
   UNLIMITED : constant TRational := (Numer=>OneComp,Denom => NilComp);
   NegativeUNLIMITED : constant TRational:=(Numer=>-OneComp,Denom=>NilComp);
   Undefined : constant TRational:=(Numer=>NilComp,Denom=>NilComp);

end Rational;